@@ -15,7 +15,9 @@ from yellowdog_client.model import (
     ComputeSourceTemplate,
     ComputeSourceTemplateSummary,
     ConfiguredWorkerPool,
+    Instance,
     KeyringSummary,
+    MachineImageFamilySearch,
     MachineImageFamilySummary,
     NamespaceStorageConfiguration,
     ObjectPath,
@@ -28,6 +30,8 @@ from yellowdog_client.model import (
     WorkRequirementSummary,
 )
 
+from yd_commands.aws_types import AWSAvailabilityZone
+
 
 @lru_cache()
 def get_task_groups_from_wr_summary(
@@ -80,11 +84,14 @@ def get_filtered_work_requirements(
 
     for work_summary in work_requirement_summaries:
         work_summary.tag = "" if work_summary.tag is None else work_summary.tag
+        work_summary.namespace = (
+            "" if work_summary.namespace is None else work_summary.namespace
+        )
         if (
             work_summary.status in include_filter
             or not work_summary.status in exclude_filter
-            and work_summary.namespace == namespace
-            and work_summary.tag.startswith(tag)
+            and namespace in work_summary.namespace
+            and tag in work_summary.tag
         ):
             filtered_work_summaries.append(work_summary)
 
@@ -98,6 +105,7 @@ Item = TypeVar(
     ComputeRequirementTemplateSummary,
     ComputeSourceTemplate,
     ComputeSourceTemplateSummary,
+    Instance,
     MachineImageFamilySummary,
     KeyringSummary,
     NamespaceStorageConfiguration,
@@ -107,6 +115,7 @@ Item = TypeVar(
     TaskGroup,
     WorkerPoolSummary,
     WorkRequirementSummary,
+    AWSAvailabilityZone,
 )
 
 
@@ -171,17 +180,20 @@ def get_work_requirement_summary_by_name_or_id(
             return work_requirement_summary
 
 
-def find_compute_source_id_by_name(client: PlatformClient, name: str) -> Optional[str]:
+def find_compute_source_template_id_by_name(
+    client: PlatformClient, name: str
+) -> Optional[str]:
     """
-    Find a compute source id by its name.
+    Find a Compute Source Template id by name.
+    Compute Source Template names are unique.
     """
-    for source in get_all_compute_sources(client):
+    for source in get_all_compute_source_templates(client):
         if source.name == name:
             return source.id
 
 
-@lru_cache
-def get_all_compute_sources(
+@lru_cache()
+def get_all_compute_source_templates(
     client: PlatformClient,
 ) -> List[ComputeSourceTemplateSummary]:
     """
@@ -190,22 +202,80 @@ def get_all_compute_sources(
     return client.compute_client.find_all_compute_source_templates()
 
 
-def find_compute_template_id_by_name(
+def clear_compute_source_template_cache():
+    """
+    Clear the cache of Compute Source Templates.
+    """
+    get_all_compute_source_templates.cache_clear()
+
+
+def find_compute_requirement_template_ids_by_name(
     client: PlatformClient, name: str
-) -> Optional[str]:
+) -> List[str]:
     """
-    Find a Compute Template ID by name.
+    Find Compute Template IDs that match the provided name.
+    (Compute Requirement Template names don't currently need
+    to be unique.)
     """
-    for template in get_all_compute_templates(client):
+    compute_template_ids = []
+    for template in get_all_compute_requirement_templates(client):
         if template.name == name:
-            return template.id
+            compute_template_ids.append(template.id)
+    return compute_template_ids
 
 
-@lru_cache
-def get_all_compute_templates(
+@lru_cache()
+def get_all_compute_requirement_templates(
     client: PlatformClient,
 ) -> List[ComputeRequirementTemplateSummary]:
     """
-    Cache the list of Compute Templates
+    Cache the list of Compute Requirement Templates.
     """
     return client.compute_client.find_all_compute_requirement_templates()
+
+
+def clear_compute_requirement_template_cache():
+    """
+    Clear the cache of Compute Requirement Templates.
+    """
+    get_all_compute_requirement_templates.cache_clear()
+
+
+def get_compute_requirement_id_by_worker_pool_id(
+    client: PlatformClient, worker_pool_id: str
+) -> Optional[str]:
+    """
+    Get a Compute Requirement ID from a Provisioned Worker Pool ID.
+    """
+    worker_pool: WorkerPool = client.worker_pool_client.get_worker_pool_by_id(
+        worker_pool_id
+    )
+    if isinstance(worker_pool, ProvisionedWorkerPool):
+        return worker_pool.computeRequirementId
+
+
+@lru_cache()
+def find_image_family_ids_by_name(
+    client: PlatformClient, image_family_name
+) -> List[str]:
+    """
+    Find image family IDs by their name.
+    """
+    if_search = MachineImageFamilySearch(
+        familyName=image_family_name, includePublic=True
+    )
+    search_client: SearchClient = client.images_client.get_image_families(if_search)
+    image_families: List[MachineImageFamilySummary] = search_client.list_all()
+
+    return [
+        image_family.id
+        for image_family in image_families
+        if image_family.name == image_family_name
+    ]
+
+
+def clear_image_family_search_cache():
+    """
+    Clear the cache of Image Family name searches.
+    """
+    find_image_family_ids_by_name.cache_clear()
